@@ -13,8 +13,7 @@ def singleton(cls):
     """
     This decorator can be used to create a singleton out of a class.
 
-    Usage::
-
+    Usage:
         @singleton
         class MySingleton():
 
@@ -63,14 +62,11 @@ def cached_class(klass: type[Klass]) -> type[Klass]:
 
         def __new__(cls, *args, **kwargs):
             """
-            Pass through...
-            :param args:
-            :param kwargs:
-            :return:
+            Pass through.
             """
-            key = (cls,) + args + tuple(kwargs.items())
+            key = (cls, *args, *tuple(kwargs.items()))
             try:
-                inst = cache.get(key, None)
+                inst = cache.get(key)
             except TypeError:
                 # Can't cache this set of arguments
                 inst = key = None
@@ -107,7 +103,7 @@ class NullFile:
 
     def __new__(cls):
         """
-        Pass through
+        Pass through.
         """
         return open(os.devnull, "w")  # pylint: disable=R1732
 
@@ -121,5 +117,4 @@ class NullStream:
     def write(self, *args):  # pylint: disable=E0211
         """
         Does nothing...
-        :param args:
         """
