@@ -2,8 +2,10 @@
 Multiprocessing utilities.
 """
 
+from __future__ import annotations
+
 from multiprocessing import Pool
-from typing import Iterable, Callable
+from typing import Callable, Iterable
 
 try:
     from tqdm.autonotebook import tqdm
@@ -11,17 +13,20 @@ except ImportError:
     tqdm = None
 
 
-def imap_tqdm(nprocs: int, func: Callable, iterable: Iterable, *args, **kwargs):
+def imap_tqdm(nprocs: int, func: Callable, iterable: Iterable, *args, **kwargs) -> list:
     """
     A wrapper around Pool.imap. Creates a Pool with nprocs and then runs a f
     unction over an iterable with progress bar.
 
-    :param nprocs: Number of processes
-    :param func: Callable
-    :param iterable: Iterable of arguments.
-    :param args: Passthrough to Pool.imap
-    :param kwargs: Passthrough to Pool.imap
-    :return: Results of Pool.imap.
+    Args:
+        nprocs: Number of processes
+        func: Callable
+        iterable: Iterable of arguments.
+        args: Passthrough to Pool.imap
+        kwargs: Passthrough to Pool.imap
+
+    Returns:
+        Results of Pool.imap.
     """
     if tqdm is None:
         raise ImportError("tqdm must be installed for this function.")
@@ -31,8 +36,8 @@ def imap_tqdm(nprocs: int, func: Callable, iterable: Iterable, *args, **kwargs):
             n = len(iterable)  # type: ignore
         except TypeError:
             n = None  # type: ignore
-        with tqdm(total=n) as pbar:
-            for i, d in enumerate(pool.imap(func, iterable, *args, **kwargs)):
-                pbar.update()
+        with tqdm(total=n) as prog_bar:
+            for d in pool.imap(func, iterable, *args, **kwargs):
+                prog_bar.update()
                 data.append(d)
     return data
