@@ -1,4 +1,5 @@
 """Copying and zipping utilities. Works on directories mostly."""
+
 from __future__ import annotations
 
 import os
@@ -6,9 +7,12 @@ import shutil
 import warnings
 from gzip import GzipFile
 from pathlib import Path
-from typing import Literal
+from typing import TYPE_CHECKING
 
-from .io import zopen
+from monty.io import zopen
+
+if TYPE_CHECKING:
+    from typing import Literal, Optional
 
 
 def copy_r(src: str | Path, dst: str | Path) -> None:
@@ -26,12 +30,14 @@ def copy_r(src: str | Path, dst: str | Path) -> None:
     abssrc = src.resolve()
     absdst = dst.resolve()
     os.makedirs(absdst, exist_ok=True)
-    for f in os.listdir(abssrc):
-        fpath = Path(abssrc, f)
-        if Path(fpath).is_file():
+    for filepath in os.listdir(abssrc):
+        fpath = Path(abssrc, filepath)
+        if fpath.is_symlink():
+            continue
+        if fpath.is_file():
             shutil.copy(fpath, absdst)
         elif str(fpath) not in str(absdst):
-            copy_r(fpath, Path(absdst, f))
+            copy_r(fpath, Path(absdst, filepath))
         else:
             warnings.warn(f"Cannot copy {fpath} to itself")
 
@@ -53,16 +59,21 @@ def gzip_dir(path: str | Path, compresslevel: int = 6) -> None:
         for f in files:
             full_f = Path(root, f).resolve()
             if Path(f).suffix.lower() != ".gz" and not full_f.is_dir():
-                with open(full_f, "rb") as f_in, GzipFile(
-                    f"{full_f}.gz", "wb", compresslevel=compresslevel
-                ) as f_out:
+                with (
+                    open(full_f, "rb") as f_in,
+                    GzipFile(
+                        f"{full_f}.gz", "wb", compresslevel=compresslevel
+                    ) as f_out,
+                ):
                     shutil.copyfileobj(f_in, f_out)
                 shutil.copystat(full_f, f"{full_f}.gz")
                 os.remove(full_f)
 
 
 def compress_file(
-    filepath: str | Path, compression: Literal["gz", "bz2"] = "gz"
+    filepath: str | Path,
+    compression: Literal["gz", "bz2"] = "gz",
+    target_dir: Optional[str | Path] = None,
 ) -> None:
     """
     Compresses a file with the correct extension. Functions like standard
@@ -73,15 +84,26 @@ def compress_file(
         filepath (str | Path): Path to file.
         compression (str): A compression mode. Valid options are "gz" or
             "bz2". Defaults to "gz".
+        target_dir (str | Path): An optional target dir where the result compressed
+            file would be stored. Defaults to None for in-place compression.
     """
     filepath = Path(filepath)
-    if compression not in ["gz", "bz2"]:
+    target_dir = Path(target_dir) if target_dir is not None else None
+
+    if compression not in {"gz", "bz2"}:
         raise ValueError("Supported compression formats are 'gz' and 'bz2'.")
-    if filepath.suffix.lower() != f".{compression}":
-        with open(filepath, "rb") as f_in, zopen(
-            f"{filepath}.{compression}", "wb"
-        ) as f_out:
+
+    if filepath.suffix.lower() != f".{compression}" and not filepath.is_symlink():
+        if target_dir is not None:
+            os.makedirs(target_dir, exist_ok=True)
+            compressed_file: str | Path = target_dir / f"{filepath.name}.{compression}"
+
+        else:
+            compressed_file = f"{str(filepath)}.{compression}"
+
+        with open(filepath, "rb") as f_in, zopen(compressed_file, "wb") as f_out:
             f_out.writelines(f_in)
+
         os.remove(filepath)
 
 
@@ -101,26 +123,40 @@ def compress_dir(path: str | Path, compression: Literal["gz", "bz2"] = "gz") ->
         for f in files:
             compress_file(Path(parent, f), compression=compression)
 
-    return None
+    return
 
 
-def decompress_file(filepath: str | Path) -> str | None:
+def decompress_file(
+    filepath: str | Path, target_dir: Optional[str | Path] = None
+) -> str | None:
     """
     Decompresses a file with the correct extension. Automatically detects
     gz, bz2 or z extension.
 
     Args:
-        filepath (str): Path to file.
+        filepath (str | Path): Path to file.
+        target_dir (str | Path): An optional target dir where the result decompressed
+            file would be stored. Defaults to None for in-place decompression.
 
     Returns:
-        str: The decompressed file path.
+        str | None: The decompressed file path, None if no operation.
     """
     filepath = Path(filepath)
+    target_dir = Path(target_dir) if target_dir is not None else None
     file_ext = filepath.suffix
-    if file_ext.lower() in [".bz2", ".gz", ".z"] and filepath.is_file():
-        decompressed_file = Path(str(filepath).removesuffix(file_ext))
+
+    if file_ext.lower() in {".bz2", ".gz", ".z"} and filepath.is_file():
+        if target_dir is not None:
+            os.makedirs(target_dir, exist_ok=True)
+            decompressed_file: str | Path = target_dir / filepath.name.removesuffix(
+                file_ext
+            )
+        else:
+            decompressed_file = str(filepath).removesuffix(file_ext)
+
         with zopen(filepath, "rb") as f_in, open(decompressed_file, "wb") as f_out:
             f_out.writelines(f_in)
+
         os.remove(filepath)
 
         return str(decompressed_file)
