@@ -6,14 +6,43 @@ import json
 import os
 import pathlib
 from enum import Enum
+from typing import Union
+
+try:
+    import numpy as np
+except ImportError:
+    np = None
+
+try:
+    import pandas as pd
+except ImportError:
+    pd = None
+
+try:
+    import torch
+except ImportError:
+    torch = None
+
+try:
+    import pydantic
+except ImportError:
+    pydantic = None
+
+try:
+    from bson.objectid import ObjectId
+except ImportError:
+    ObjectId = None
 
-import numpy as np
-import pandas as pd
 import pytest
-import torch
-from bson.objectid import ObjectId
 
-from monty.json import MontyDecoder, MontyEncoder, MSONable, _load_redirect, jsanitize
+from monty.json import (
+    MontyDecoder,
+    MontyEncoder,
+    MSONable,
+    _load_redirect,
+    jsanitize,
+    load,
+)
 
 from . import __version__ as tests_version
 
@@ -40,6 +69,29 @@ class GoodMSONClass(MSONable):
         )
 
 
+class GoodNOTMSONClass:
+    """Literally the same as the GoodMSONClass, except it does not have
+    the MSONable inheritance!"""
+
+    def __init__(self, a, b, c, d=1, *values, **kwargs):
+        self.a = a
+        self.b = b
+        self._c = c
+        self._d = d
+        self.values = values
+        self.kwargs = kwargs
+
+    def __eq__(self, other):
+        return (
+            self.a == other.a
+            and self.b == other.b
+            and self._c == other._c
+            and self._d == other._d
+            and self.kwargs == other.kwargs
+            and self.values == other.values
+        )
+
+
 class LimitedMSONClass(MSONable):
     """An MSONable class that only accepts a limited number of options"""
 
@@ -56,7 +108,7 @@ class GoodNestedMSONClass(MSONable):
         assert isinstance(b_dict, dict)
         assert isinstance(c_list_dict_list, list)
         assert isinstance(c_list_dict_list[0], dict)
-        first_key = list(c_list_dict_list[0].keys())[0]
+        first_key = next(iter(c_list_dict_list[0]))
         assert isinstance(c_list_dict_list[0][first_key], list)
         self.a_list = a_list
         self.b_dict = b_dict
@@ -166,8 +218,7 @@ class TestMSONable:
                 self.b = b
 
             def as_dict(self):
-                d = {"init": {"a": self.a, "b": self.b}}
-                return d
+                return {"init": {"a": self.a, "b": self.b}}
 
         self.bad_cls = BadMSONClass
 
@@ -185,6 +236,12 @@ class TestMSONable:
 
         self.auto_mson = AutoMSON
 
+        class ClassContainingKWOnlyArgs(MSONable):
+            def __init__(self, *, a):
+                self.a = a
+
+        self.kw_only_args_cls = ClassContainingKWOnlyArgs
+
     def test_to_from_dict(self):
         obj = self.good_cls("Hello", "World", "Python")
         d = obj.as_dict()
@@ -205,6 +262,16 @@ class TestMSONable:
         d = obj.as_dict()
         self.auto_mson.from_dict(d)
 
+    def test_kw_only_args(self):
+        obj = self.kw_only_args_cls(a=1)
+        d = obj.as_dict()
+        assert d is not None
+        assert d["a"] == 1
+        self.kw_only_args_cls.from_dict(d)
+        jsonstr = obj.to_json()
+        d = json.loads(jsonstr)
+        assert d["@class"], "ClassContainingKWOnlyArgs"
+
     def test_unsafe_hash(self):
         GMC = GoodMSONClass
         a_list = [GMC(1, 1.0, "one"), GMC(2, 2.0, "two")]
@@ -302,7 +369,9 @@ class TestMSONable:
 
         d = {"123": EnumTest.a}
         f = jsanitize(d)
-        assert f["123"] == "EnumTest.a"
+        assert f["123"]["@module"] == "tests.test_json"
+        assert f["123"]["@class"] == "EnumTest"
+        assert f["123"]["value"] == 1
 
         f = jsanitize(d, strict=True)
         assert f["123"]["@module"] == "tests.test_json"
@@ -315,6 +384,86 @@ class TestMSONable:
         f = jsanitize(d, enum_values=True)
         assert f["123"] == 1
 
+    def test_enum_serialization_no_msonable(self):
+        d = {"123": EnumNoAsDict.name_a}
+        f = jsanitize(d)
+        assert f["123"]["@module"] == "tests.test_json"
+        assert f["123"]["@class"] == "EnumNoAsDict"
+        assert f["123"]["value"] == "value_a"
+
+        f = jsanitize(d, strict=True)
+        assert f["123"]["@module"] == "tests.test_json"
+        assert f["123"]["@class"] == "EnumNoAsDict"
+        assert f["123"]["value"] == "value_a"
+
+        f = jsanitize(d, strict=True, enum_values=True)
+        assert f["123"] == "value_a"
+
+        f = jsanitize(d, enum_values=True)
+        assert f["123"] == "value_a"
+
+    def test_save_load(self, tmp_path):
+        """Tests the save and load serialization methods."""
+
+        test_good_class = GoodMSONClass(
+            "Hello",
+            "World",
+            "Python",
+            **{
+                "cant_serialize_me": GoodNOTMSONClass(
+                    "Hello2", "World2", "Python2", **{"values": []}
+                ),
+                "cant_serialize_me2": [
+                    GoodNOTMSONClass("Hello4", "World4", "Python4", **{"values": []}),
+                    GoodNOTMSONClass("Hello4", "World4", "Python4", **{"values": []}),
+                ],
+                "cant_serialize_me3": [
+                    {
+                        "tmp": GoodMSONClass(
+                            "Hello5", "World5", "Python5", **{"values": []}
+                        ),
+                        "tmp2": 2,
+                        "tmp3": [1, 2, 3],
+                    },
+                    {
+                        "tmp5": GoodNOTMSONClass(
+                            "aHello5", "aWorld5", "aPython5", **{"values": []}
+                        ),
+                        "tmp2": 5,
+                        "tmp3": {"test": "test123"},
+                    },
+                    # Gotta check that if I hide an MSONable class somewhere
+                    # it still gets correctly serialized.
+                    {"actually_good": GoodMSONClass("1", "2", "3", **{"values": []})},
+                ],
+                "values": [],
+            },
+        )
+
+        # This will pass
+        test_good_class.as_dict()
+
+        # This will fail
+        with pytest.raises(TypeError):
+            test_good_class.to_json()
+
+        # This should also pass though
+        target = tmp_path / "test.json"
+        test_good_class.save(target, json_kwargs={"indent": 4, "sort_keys": True})
+
+        # This will fail
+        with pytest.raises(FileExistsError):
+            test_good_class.save(target, strict=True)
+
+        # Now check that reloading this, the classes are equal!
+        test_good_class2 = GoodMSONClass.load(target)
+
+        # Final check using load
+        test_good_class3 = load(target)
+
+        assert test_good_class == test_good_class2
+        assert test_good_class == test_good_class3
+
 
 class TestJson:
     def test_as_from_dict(self):
@@ -338,6 +487,7 @@ class TestJson:
         listobj2 = json.loads(s, cls=MontyDecoder)
         assert listobj2[0].a.a == 1
 
+    @pytest.mark.skipif(torch is None, reason="torch not present")
     def test_torch(self):
         t = torch.tensor([0, 1, 2])
         jsonstr = json.dumps(t, cls=MontyEncoder)
@@ -380,12 +530,22 @@ class TestJson:
         d = json.loads(jsonstr, cls=MontyDecoder)
         assert isinstance(d["uuid"], UUID)
 
+    def test_path(self):
+        from pathlib import Path
+
+        p = Path("/home/user/")
+        jsonstr = json.dumps(p, cls=MontyEncoder)
+        d = json.loads(jsonstr, cls=MontyDecoder)
+        assert isinstance(d, Path)
+        assert d == p
+
     def test_nan(self):
         x = [float("NaN")]
         djson = json.dumps(x, cls=MontyEncoder)
         d = json.loads(djson)
         assert isinstance(d[0], float)
 
+    @pytest.mark.skipif(np is None, reason="numpy not present")
     def test_numpy(self):
         x = np.array([1, 2, 3], dtype="int64")
         with pytest.raises(TypeError):
@@ -432,6 +592,10 @@ class TestJson:
         d = jsanitize(x, strict=True)
         assert isinstance(d["energies"][0], float)
 
+        x = {"energy": np.array(-1.0)}
+        d = jsanitize(x, strict=True)
+        assert isinstance(d["energy"], float)
+
         # Test data nested in a class
         x = np.array([[1 + 1j, 2 + 1j], [3 + 1j, 4 + 1j]], dtype="complex64")
         cls = ClassContainingNumpyArray(np_a={"a": [{"b": x}]})
@@ -451,6 +615,7 @@ class TestJson:
         assert isinstance(obj.np_a["a"][0]["b"], np.ndarray)
         assert obj.np_a["a"][0]["b"][0][1] == 2 + 1j
 
+    @pytest.mark.skipif(pd is None, reason="pandas not present")
     def test_pandas(self):
         cls = ClassContainingDataFrame(
             df=pd.DataFrame([{"a": 1, "b": 1}, {"a": 1, "b": 2}])
@@ -549,6 +714,7 @@ class TestJson:
         djson = json.dumps(instance, cls=MontyEncoder)
         assert "@class" in djson
 
+    @pytest.mark.skipif(ObjectId is None, reason="bson not present")
     def test_objectid(self):
         oid = ObjectId("562e8301218dcbbc3d7d91ce")
         with pytest.raises(TypeError):
@@ -564,7 +730,7 @@ class TestJson:
         assert clean["world"] is None
         assert json.loads(json.dumps(d)) == json.loads(json.dumps(clean))
 
-        d = {"hello": GoodMSONClass(1, 2, 3)}
+        d = {"hello": GoodMSONClass(1, 2, 3), "test": "hi"}
         with pytest.raises(TypeError):
             json.dumps(d)
         clean = jsanitize(d)
@@ -572,9 +738,28 @@ class TestJson:
         clean_strict = jsanitize(d, strict=True)
         assert clean_strict["hello"]["a"] == 1
         assert clean_strict["hello"]["b"] == 2
+        assert clean_strict["test"] == "hi"
         clean_recursive_msonable = jsanitize(d, recursive_msonable=True)
         assert clean_recursive_msonable["hello"]["a"] == 1
         assert clean_recursive_msonable["hello"]["b"] == 2
+        assert clean_recursive_msonable["hello"]["c"] == 3
+        assert clean_recursive_msonable["test"] == "hi"
+
+        d = {"hello": [GoodMSONClass(1, 2, 3), "test"], "test": "hi"}
+        clean_recursive_msonable = jsanitize(d, recursive_msonable=True)
+        assert clean_recursive_msonable["hello"][0]["a"] == 1
+        assert clean_recursive_msonable["hello"][0]["b"] == 2
+        assert clean_recursive_msonable["hello"][0]["c"] == 3
+        assert clean_recursive_msonable["hello"][1] == "test"
+        assert clean_recursive_msonable["test"] == "hi"
+
+        d = {"hello": (GoodMSONClass(1, 2, 3), "test"), "test": "hi"}
+        clean_recursive_msonable = jsanitize(d, recursive_msonable=True)
+        assert clean_recursive_msonable["hello"][0]["a"] == 1
+        assert clean_recursive_msonable["hello"][0]["b"] == 2
+        assert clean_recursive_msonable["hello"][0]["c"] == 3
+        assert clean_recursive_msonable["hello"][1] == "test"
+        assert clean_recursive_msonable["test"] == "hi"
 
         d = {"dt": datetime.datetime.now()}
         clean = jsanitize(d)
@@ -582,14 +767,6 @@ class TestJson:
         clean = jsanitize(d, allow_bson=True)
         assert isinstance(clean["dt"], datetime.datetime)
 
-        d = {
-            "a": ["b", np.array([1, 2, 3])],
-            "b": ObjectId.from_datetime(datetime.datetime.now()),
-        }
-        clean = jsanitize(d)
-        assert clean["a"], ["b", [1, 2 == 3]]
-        assert isinstance(clean["b"], str)
-
         rnd_bin = bytes(np.random.rand(10))
         d = {"a": bytes(rnd_bin)}
         clean = jsanitize(d, allow_bson=True)
@@ -652,18 +829,31 @@ class TestJson:
         clean = jsanitize(d, strict=True)
         assert "@class" in clean["c"]
 
-        # test on pandas
-        df = pd.DataFrame([{"a": 1, "b": 1}, {"a": 1, "b": 2}])
-        clean = jsanitize(df)
-        assert clean == df.to_dict()
-
+    @pytest.mark.skipif(pd is None, reason="pandas not present")
+    def test_jsanitize_pandas(self):
         s = pd.Series({"a": [1, 2, 3], "b": [4, 5, 6]})
         clean = jsanitize(s)
         assert clean == s.to_dict()
 
+    @pytest.mark.skipif(
+        np is None or ObjectId is None, reason="numpy and bson not present"
+    )
+    def test_jsanitize_numpy_bson(self):
+        d = {
+            "a": ["b", np.array([1, 2, 3])],
+            "b": ObjectId.from_datetime(datetime.datetime.now()),
+        }
+        clean = jsanitize(d)
+        assert clean["a"], ["b", [1, 2 == 3]]
+        assert isinstance(clean["b"], str)
+
     def test_redirect(self):
         MSONable.REDIRECT["tests.test_json"] = {
-            "test_class": {"@class": "GoodMSONClass", "@module": "tests.test_json"}
+            "test_class": {"@class": "GoodMSONClass", "@module": "tests.test_json"},
+            "another_test_class": {
+                "@class": "AnotherClass",
+                "@module": "tests.test_json2",
+            },
         }
 
         d = {
@@ -677,9 +867,18 @@ class TestJson:
         obj = json.loads(json.dumps(d), cls=MontyDecoder)
         assert isinstance(obj, GoodMSONClass)
 
-        d["@class"] = "not_there"
-        obj = json.loads(json.dumps(d), cls=MontyDecoder)
-        assert isinstance(obj, dict)
+        d2 = {
+            "@class": "another_test_class",
+            "@module": "tests.test_json",
+            "a": 2,
+            "b": 2,
+            "c": 2,
+        }
+
+        with pytest.raises(ImportError, match="No module named 'tests.test_json2'"):
+            # This should raise ImportError because it's trying to load
+            # AnotherClass from tests.test_json instead of tests.test_json2
+            json.loads(json.dumps(d2), cls=MontyDecoder)
 
     def test_redirect_settings_file(self):
         data = _load_redirect(os.path.join(test_dir, "test_settings.yaml"))
@@ -689,6 +888,7 @@ class TestJson:
             }
         }
 
+    @pytest.mark.skipif(pydantic is None, reason="pydantic not present")
     def test_pydantic_integrations(self):
         from pydantic import BaseModel, ValidationError
 
@@ -788,7 +988,7 @@ class TestJson:
             a: LimitedMSONClass
 
         class ModelWithUnion(BaseModel):
-            a: LimitedMSONClass | dict
+            a: Union[LimitedMSONClass, dict]
 
         limited_dict = jsanitize(ModelWithLimited(a=LimitedMSONClass(1)), strict=True)
         assert ModelWithLimited.model_validate(limited_dict)
