@@ -1,6 +1,7 @@
+from __future__ import annotations
+
 import os
 import shutil
-import unittest
 
 from monty.tempfile import ScratchDir
 
@@ -49,14 +50,16 @@ class TestScratchDir:
         with open("pre_scratch_text", "w") as f:
             f.write("write")
         init_gz = [f for f in os.listdir(os.getcwd()) if f.endswith(".gz")]
-        with ScratchDir(
-            self.scratch_root,
-            copy_from_current_on_enter=True,
-            copy_to_current_on_exit=True,
-            gzip_on_exit=True,
+        with (
+            ScratchDir(
+                self.scratch_root,
+                copy_from_current_on_enter=True,
+                copy_to_current_on_exit=True,
+                gzip_on_exit=True,
+            ),
+            open("scratch_text", "w") as f,
         ):
-            with open("scratch_text", "w") as f:
-                f.write("write")
+            f.write("write")
         files = os.listdir(os.getcwd())
 
         # Make sure the stratch_text.gz exists
@@ -143,7 +146,3 @@ class TestScratchDir:
     def teardown_method(self):
         os.chdir(self.cwd)
         shutil.rmtree(self.scratch_root)
-
-
-if __name__ == "__main__":
-    unittest.main()
